@@ -6,8 +6,13 @@ make_col_names <- function(n, is.dynamic, prefix="v.") {
 }
 
 # Checks attributes to be added to a diffnet object
-check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL,
-                                   attr.class="vertex") {
+check_as_diffnet_attrs <- function(
+  attrs,
+  meta, is.dynamic,
+  id.and.per.vars=NULL,
+  attr.class="vertex"
+  ) {
+
   # Getting meta
   n    <- meta$n
   nper <- meta$nper
@@ -321,16 +326,18 @@ check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL
 
 #' Creates a \code{diffnet} class object
 #'
-#' \code{diffnet} objects contain difussion networks. With adjacency
-#' matrices and time of adoption (toa) vector as its main components, most of the
-#' package's functions have methods for this class of objects.
+#' \code{diffnet} objects contain diffusion networks. With adjacency
+#' matrices and time of adoption (toa) vector (or matrix, for multiple behavior diffusion),
+#' as its main components, most of the package's functions have methods for this class of objects.
 #'
 #' @templateVar dynamic TRUE
 #' @templateVar undirected TRUE
 #' @templateVar self TRUE
 #' @templateVar multiple TRUE
 #' @template graph_template
-#' @param toa Numeric vector of size \eqn{n}. Times of adoption.
+#' @param toa Numeric vector of size \eqn{n}. Times of adoption. For \eqn{Q}{Q}
+#' multiple behavior diffusion, \code{toa} must be a matrix \eqn{n \times Q}{n * Q}
+#' (see \code{\link{rdiffnet}}, examples of multiple behavior diffusion).
 #' @param t0 Integer scalar. Passed to \code{\link{toa_mat}}.
 #' @param t1 Integer scalar. Passed to \code{\link{toa_mat}}.
 #' @param ... Further arguments passed to the jmethod.
@@ -348,7 +355,7 @@ check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL
 #' or static (\code{"static"}).
 #' @param as.df Logical scalar. When TRUE returns a data.frame.
 #' @param name Character scalar. Name of the diffusion network (descriptive).
-#' @param behavior Character scalar. Name of the behavior been analyzed (innovation).
+#' @param behavior Character vector. Name of the behavior(s) been analyzed (innovation).
 #'
 #' @seealso Default options are listed at \code{\link{netdiffuseR-options}}
 #' @details
@@ -418,6 +425,8 @@ check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL
 #' @aliases diffnet diffnet-class
 #' @examples
 #'
+#' # Creating a diffnet object from TOA (time of adoption) ---------------------
+#'
 #' # Creating a random graph
 #' set.seed(123)
 #' graph <- rgraph_ba(t=9)
@@ -435,6 +444,16 @@ check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL
 #' # Plotting slice 4
 #' plot(diffnet, t=4)
 #'
+#' # A diffnet object from TOA of multiple behaviors ---------------------------
+#'
+#' # TOA for two behaviors
+#' toa_matrix <- matrix(sample(c(2001L:2005L,NA), 20, TRUE), ncol = 2)
+#'
+#' # Creating diffnet object
+#' diffnet_multi <- new_diffnet(graph, toa_matrix)
+#' diffnet_multi
+#' summary(diffnet_multi)
+#'
 #' # ATTRIBUTES ----------------------------------------------------------------
 #'
 #' # Retrieving attributes
@@ -491,9 +510,11 @@ check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL
 #' A list of class \code{diffnet} with the following elements:
 #' \item{graph}{A list of length \eqn{T}. Containing sparse square matrices of size \eqn{n}
 #' and class \code{\link[Matrix:dgCMatrix-class]{dgCMatrix}}.}
-#' \item{toa}{An integer vector of size \eqn{T} with times of adoption.}
+#' \item{toa}{An integer vector of length \eqn{n} with times of adoption. When \eqn{Q}{Q} multiple
+#' behavior diffusion is selected, a matrix of size \eqn{n \times Q}{n * Q}}.
 #' \item{adopt, cumadopt}{Numeric matrices of size \eqn{n\times T}{n*T} as those returned
-#' by \code{\link{toa_mat}}.}
+#' by \code{\link{toa_mat}}. For \eqn{Q}{Q} multiple behavior diffusion, \code{adopt} and \code{cumadopt}
+#' become a list of \eqn{n\times T}{n*T} elements, with \eqn{Q}{Q} elements.}
 #' \item{vertex.static.attrs}{If not NULL, a data frame with \eqn{n} rows with vertex static
 #' attributes.}
 #' \item{vertex.dyn.attrs}{A list of length \eqn{T} with data frames containing vertex attributes
@@ -511,10 +532,10 @@ check_as_diffnet_attrs <- function(attrs, meta, is.dynamic, id.and.per.vars=NULL
 #'  \item \code{undirected}: Logical scalar.
 #'  \item \code{multiple}: Logical scalar.
 #'  \item \code{name}: Character scalar.
-#'  \item \code{behavior}: Character scalar.
+#'  \item \code{behavior}: A list of character scalars.
 #' }
 #' }
-#' @author George G. Vega Yon
+#' @author George G. Vega Yon & Aníbal Olivera M.
 #' @name diffnet-class
 NULL
 
@@ -539,16 +560,20 @@ as_diffnet.networkDynamic <- function(graph, toavar, ...) {
 
 #' @export
 #' @rdname diffnet-class
-new_diffnet <- function(graph, toa, t0=min(toa, na.rm = TRUE), t1=max(toa, na.rm = TRUE),
-                       vertex.dyn.attrs = NULL, vertex.static.attrs= NULL,
-                       id.and.per.vars = NULL,
-                       graph.attrs = NULL,
-                       undirected = getOption("diffnet.undirected"),
-                       self       = getOption("diffnet.self"),
-                       multiple   = getOption("diffnet.multiple"),
-                       name       = "Diffusion Network",
-                       behavior   = "Unspecified"
-                       ) {
+new_diffnet <- function(
+  graph, toa,
+  t0=min(toa, na.rm   = TRUE),
+  t1=max(toa, na.rm   = TRUE),
+  vertex.dyn.attrs    = NULL,
+  vertex.static.attrs = NULL,
+  id.and.per.vars     = NULL,
+  graph.attrs         = NULL,
+  undirected          = getOption("diffnet.undirected"),
+  self                = getOption("diffnet.self"),
+  multiple            = getOption("diffnet.multiple"),
+  name                = "Diffusion Network",
+  behavior            = NULL
+) {
 
   # Step 0.0: Check if its diffnet! --------------------------------------------
   if (inherits(graph, "diffnet")) {
@@ -556,6 +581,22 @@ new_diffnet <- function(graph, toa, t0=min(toa, na.rm = TRUE), t1=max(toa, na.rm
     return(graph)
   }
 
+  # Step 0.1: Setting num_of_behavior ------------------------------------------
+
+  if (inherits(toa, "matrix"))
+    num_of_behaviors <- dim(toa)[2]
+  else
+    num_of_behaviors <- 1
+
+  if (length(behavior) == 0L)
+    behavior <- rep("Unknown", num_of_behaviors)
+  else if (length(behavior) != num_of_behaviors)
+    stop(
+      "Length of -behavior- must be equal to the number of behaviors in -toa-."
+      )
+  else if (!inherits(behavior, "list"))
+    behavior <- as.list(behavior)
+
   # Step 1.1: Check graph ------------------------------------------------------
   meta <- classify_graph(graph)
   if (meta$type=="static")
@@ -563,42 +604,108 @@ new_diffnet <- function(graph, toa, t0=min(toa, na.rm = TRUE), t1=max(toa, na.rm
 
 
   # Step 1.2: Checking that lengths fit
-  if (length(toa)!=meta$n) stop("-graph- and -toa- have different lengths (",
-                                meta$n, " and ", length(toa), " respectively). ",
-                                "-toa- should be of length n (number of vertices).")
+  if ((num_of_behaviors == 1L) && (length(toa) != meta$n)) {
 
-  # Step 2.1: Checking class of TOA and coercing if necesary -------------------
+    stop(
+      "-graph- and -toa- have different lengths (", meta$n, " and ",
+      length(toa),
+      " respectively). -toa- should be of length n (number of vertices)."
+      )
+
+  } else if ((num_of_behaviors > 1L) && length(toa[, 1L])!=meta$n) {
+
+    stop(
+      "-graph- and -toa[, 1]- have different lengths (", meta$n, " and ",
+      length(toa[, 1L]),
+      " respectively). ", "-toa- should be of length n (number of vertices)."
+      )
+
+  }
+
+  # Step 2.1: Checking class of TOA and coercing if necessary
   if (!inherits(toa, "integer")) {
+
     warning("Coercing -toa- into integer.")
-    toa <- as.integer(toa)
+    toa[] <- as.integer(toa)
+
   }
 
   # Step 2.2: Checking names of toa
-  if (!length(names(toa)))
-    names(toa) <- meta$ids
+  if (num_of_behaviors==1) {
+    if (!length(names(toa))) {names(toa) <- meta$ids}
+  } else {
+    if (!length(rownames(toa))) { # Not necessary? toa_mat(toa, labels = meta$ids, t0=t0, t1=t1) already has labels
+      rownames(toa) <- meta$ids
+    }
+  }
 
   # Step 3.1: Creating Time of adoption matrix ---------------------------------
   mat <- toa_mat(toa, labels = meta$ids, t0=t0, t1=t1)
 
   # Step 3.2: Verifying dimensions and fixing meta$pers
 
-  if (meta$type != "static") {
-    tdiff <- meta$nper - ncol(mat[[1]])
-    if (tdiff < 0)
-      stop("Range of -toa- is bigger than the number of slices in -graph- (",
-           ncol(mat[[1]]), " and ", length(graph) ," respectively). ",
-           "There must be at least as many slices as range of toa.")
-    else if (tdiff > 0)
-      stop("Range of -toa- is smaller than the number of slices in -graph- (",
-           ncol(mat[[1]]), " and ", length(graph) ," respectively). ",
-           "Please provide lower and upper boundaries for the values in -toa- ",
-           "using -t0- and -t- (see ?toa_mat).")
+  if (num_of_behaviors==1) {
+    if (meta$type != "static") {
+
+      tdiff <- meta$nper - ncol(mat$adopt)
+      if (tdiff < 0)
+        stop(
+          "Range of -toa- is bigger than the number of slices in -graph- (",
+          ncol(mat$adopt), " and ", length(graph) ," respectively). ",
+          "There must be at least as many slices as range of toa."
+          )
+      else if (tdiff > 0)
+        stop(
+          "Range of -toa- is smaller than the number of slices in -graph- (",
+          ncol(mat$adopt), " and ", length(graph) ," respectively). ",
+          "Please provide lower and upper boundaries for the values in -toa- ",
+          "using -t0- and -t- (see ?toa_mat)."
+          )
+
+    } else {
+
+      graph <- lapply(
+        1:ncol(mat$adopt), function(x) methods::as(graph, "dgCMatrix")
+        )
+
+      meta  <- classify_graph(graph)
+
+    }
   } else {
-    graph <- lapply(1:ncol(mat[[1]]), function(x) methods::as(graph, "dgCMatrix"))
-    meta  <- classify_graph(graph)
+    if (meta$type != "static") {
+
+      tdiff <- meta$nper - ncol(mat[[1]]$adopt)
+      if (tdiff < 0)
+        stop(
+          "Range of -toa- is bigger than the number of slices in -graph- (",
+          ncol(mat[[1]]$adopt), " and ", length(graph) ," respectively). ",
+          "There must be at least as many slices as range of toa."
+          )
+      else if (tdiff > 0)
+        stop(
+          "Range of -toa- is smaller than the number of slices in -graph- (",
+          ncol(mat[[1]]$adopt), " and ", length(graph) ," respectively). ",
+          "Please provide lower and upper boundaries for the values in -toa- ",
+          "using -t0- and -t- (see ?toa_mat)."
+          )
+
+    } else {
+
+      graph <- lapply(
+        1:ncol(mat[[1]]$adopt), function(x) methods::as(graph, "dgCMatrix")
+        )
+
+      meta  <- classify_graph(graph)
+
+    }
   }
 
-  meta$pers <- as.integer(colnames(mat$adopt))
+  # labels of the time periods
+  if (num_of_behaviors == 1) {
+    meta$pers <- as.integer(colnames(mat$adopt))
+  } else {
+    meta$pers <- as.integer(colnames(mat[[1]]$adopt))
+  } # same for all behaviors
 
   # Step 4.0: Checking the attributes ------------------------------------------
 
@@ -627,28 +734,59 @@ new_diffnet <- function(graph, toa, t0=min(toa, na.rm = TRUE), t1=max(toa, na.rm
   meta$self       <- self
   meta$undirected <- undirected
   meta$multiple   <- multiple
-  meta$name       <- ifelse(!length(name), "", ifelse(is.na(name), "",
-                                                      as.character(name)))
-  meta$behavior   <- ifelse(!length(behavior), "", ifelse(is.na(behavior), "",
-                                                          as.character(behavior)))
+  meta$name       <- ifelse(
+    !length(name), "", ifelse(is.na(name), "", as.character(name))
+    )
 
-  # Removing dimnames
-  graph                  <- Map(function(x) Matrix::unname(x), x=graph)
-  dimnames(toa)          <- NULL
-  dimnames(mat$adopt)    <- NULL
-  dimnames(mat$cumadopt) <- NULL
+  meta$version <- utils::packageVersion("netdiffuseR")
+  graph <- Map(function(x) Matrix::unname(x), x=graph)
+
+  if (num_of_behaviors == 1) {
+
+    meta$behavior   <- behavior
+
+    dimnames(mat$adopt)    <- NULL
+    dimnames(mat$cumadopt) <- NULL
+
+    adopt <- mat$adopt
+    cumadopt <- mat$cumadopt
+
+  } else {
+
+    meta$behavior <- behavior
+
+    for (q in 1:num_of_behaviors) {
+      dimnames(mat[[q]]$adopt)    <- NULL
+      dimnames(mat[[q]]$cumadopt) <- NULL
+    }
+
+    adopt <- list()
+    cumadopt <- list()
+
+    for (q in 1:num_of_behaviors) {
+      adopt[[q]] <- mat[[q]]$adopt
+      cumadopt[[q]] <- mat[[q]]$cumadopt
+    }
+
+  }
+
+  return(
+    structure(
+      list(
+        graph = graph,
+        toa   = toa,
+        adopt = adopt,
+        cumadopt = cumadopt,
+        # Attributes
+        vertex.static.attrs = vertex.static.attrs,
+        vertex.dyn.attrs    = vertex.dyn.attrs,
+        graph.attrs         = graph.attrs,
+        meta = meta
+      ),
+      class="diffnet"
+    )
+  )
 
-  return(structure(list(
-    graph = graph,
-    toa   = toa,
-    adopt = mat$adopt,
-    cumadopt = mat$cumadopt,
-    # Attributes
-    vertex.static.attrs = vertex.static.attrs,
-    vertex.dyn.attrs    = vertex.dyn.attrs,
-    graph.attrs         = graph.attrs,
-    meta = meta
-  ), class="diffnet"))
 }
 
 #' @export
