@@ -94,16 +94,35 @@ print.diffnet <- function(x, ...) {
                      paste(head(meta$ids, 8), collapse=", "),
                      ifelse(meta$n>8, ", ...", "") ,")")
 
+    # Computing prevalence for multi-diff
+    single <- !inherits(cumadopt, "list")
+    if (!single) {
+      prevalence_all <- character(length(cumadopt))
+      for (q in 1:length(cumadopt)) {
+        prevalence <- formatC(sum(cumadopt[[q]][,meta$nper]) / meta$n, digits = 2, format="f")
+        prevalence_all[q] <- prevalence
+      }
+      prevalence_all <- paste(prevalence_all, collapse = ", ")
+
+      num_of_behavior <- as.character(length(cumadopt))
+    } else {
+      prevalence <- formatC(sum(cumadopt[,meta$nper])/meta$n, digits = 2, format="f")
+      num_of_behavior <- "1"
+    }
+
     cat(
     "Dynamic network of class -diffnet-",
     paste(" Name               :", meta$name),
-    paste(" Behavior           :", meta$behavior),
+    paste(" Behavior           :", paste(meta$behavior, collapse=", ")),
     paste(" # of nodes         :", nodesl ),
     paste(" # of time periods  :", meta$nper, sprintf("(%d - %d)", meta$pers[1], meta$pers[meta$nper])),
     paste(" Type               :", ifelse(meta$undirected, "undirected", "directed")),
-    paste(" Final prevalence   :",
-          formatC(sum(cumadopt[,meta$nper])/meta$n, digits = 2, format="f")
-          ),
+    paste(" Num of behaviors   :", num_of_behavior),
+    if (single) {
+      paste(" Final prevalence   :", prevalence)
+    } else {
+      paste(" Prevalence         :", prevalence_all)
+    },
     paste(" Static attributes  :", vsa),
     paste(" Dynamic attributes :", vda),
     sep="\n"
@@ -163,12 +182,12 @@ print.diffnet <- function(x, ...) {
 #' @family diffnet methods
 #'
 summary.diffnet <- function(
-  object,
-  slices     = NULL,
-  no.print   = FALSE,
-  skip.moran = FALSE,
-  valued     = getOption("diffnet.valued",FALSE),
-  ...) {
+    object,
+    slices     = NULL,
+    no.print   = FALSE,
+    skip.moran = FALSE,
+    valued     = getOption("diffnet.valued",FALSE),
+    ...) {
   # Subsetting
   if (!length(slices)) slices <- 1:object$meta$nper
 
@@ -194,50 +213,103 @@ summary.diffnet <- function(
     # x <-nelements/(meta$n * (meta$n-1))
   }))
 
-  # Computing moran's I
-  if (!skip.moran) {
+  # identify single-diff from multi-diff
+  single <- !inherits(object$cumadopt, "list")
 
-    m <- matrix(NA, nrow=length(slices), ncol=4,
-                dimnames = list(NULL, c("moran_obs", "moran_exp", "moran_sd", "moran_pval")))
+  if (single) {
+    # Computing moran's I
+    if (!skip.moran) {
+      m <- matrix(NA, nrow=length(slices), ncol=4,
+                  dimnames = list(NULL, c("moran_obs", "moran_exp", "moran_sd", "moran_pval")))
 
-    for (i in 1:length(slices)) {
-      # Computing distances
-      g <- approx_geodesic(object$graph[[slices[i]]], ...)
+      for (i in 1:length(slices)) {
+        # Computing distances
+        g <- approx_geodesic(object$graph[[slices[i]]], ...)
+        # Inverting it (only the diagonal may have 0)
+        g@x <- 1/g@x
 
-      # Inverting it (only the diagonal may have 0)
-      g@x <- 1/g@x
-
-      m[i,] <- unlist(moran(object$cumadopt[,slices[i]], g))
+        m[i,] <- unlist(moran(object$cumadopt[,slices[i]], g))
+      }
     }
+
+    # Computing new adopters, cumadopt and hazard rate
+    ad <- colSums(object$adopt[,slices,drop=FALSE])
+    ca <- t(cumulative_adopt_count(object$cumadopt))[slices,-3, drop=FALSE]
+    hr <- t(hazard_rate(object$cumadopt, no.plot = TRUE))[slices,,drop=FALSE]
+
+    # Left censoring
+    lc <- sum(object$toa == meta$pers[1], na.rm = TRUE)
+    rc <- sum(is.na(object$toa), na.rm=TRUE)
+
+    out <- data.frame(
+      adopt = ad,
+      cum_adopt = ca[,1],
+      cum_adopt_pcent = ca[,2],
+      hazard = hr,
+      density=d
+    )
+
+    if (!skip.moran) {
+      out <- cbind(out, m)
+    }
+
+    if (no.print) return(out)
+
+  } else {
+
+    out_list <- list()
+    data_beh_list <- list()
+
+    for (q in 1:length(object$cumadopt)) {
+
+      if (!skip.moran) {
+        #for (q in 1:length(object$cumadopt)) {
+        m <- matrix(NA, nrow=length(slices), ncol=4,
+                    dimnames = list(NULL, c("moran_obs", "moran_exp", "moran_sd", "moran_pval")))
+
+        for (i in 1:length(slices)) {
+
+          g <- approx_geodesic(object$graph[[slices[i]]], ...)
+          g@x <- 1/g@x
+
+          m[i,] <- unlist(moran(object$cumadopt[[q]][,slices[i]], g))
+        }
+        #}
+      }
+
+      # Computing new adopters, cumadopt and hazard rate
+      ad <- colSums(object$adopt[[q]][,slices,drop=FALSE])
+      ca <- t(cumulative_adopt_count(object$cumadopt[[q]]))[slices,-3, drop=FALSE]
+      hr <- t(hazard_rate(object$cumadopt[[q]], no.plot = TRUE))[slices,,drop=FALSE]
+
+      # Left censoring
+      lc <- sum(object$toa[,q] == meta$pers[1], na.rm = TRUE)
+      rc <- sum(is.na(object$toa[,q]), na.rm=TRUE)
+
+      data_beh_list[[q]] <- c(lc, rc)
+
+      out <- data.frame(
+        adopt = ad,
+        cum_adopt = ca[,1],
+        cum_adopt_pcent = ca[,2],
+        hazard = hr,
+        density=d
+      )
+
+      if (!skip.moran) {
+        out <- cbind(out, m)
+      }
+
+      out_list[[q]] <- out
+    }
+
+    if (no.print) return(out)
   }
-  # Computing adopters, cumadopt and hazard rate
-  ad <- colSums(object$adopt[,slices,drop=FALSE])
-  ca <- t(cumulative_adopt_count(object$cumadopt))[slices,-3, drop=FALSE]
-  hr <- t(hazard_rate(object$cumadopt, no.plot = TRUE))[slices,,drop=FALSE]
-
-  # Left censoring
-  lc <- sum(object$toa == meta$pers[1], na.rm = TRUE)
-  rc <- sum(is.na(object$toa), na.rm=TRUE)
-
-  out <- data.frame(
-    adopt = ad,
-    cum_adopt = ca[,1],
-    cum_adopt_pcent = ca[,2],
-    hazard = hr,
-    density=d
-  )
-
-  if (!skip.moran) {
-    out <- cbind(out, m)
-  }
-
-  if (no.print) return(out)
 
   # Function to print data.frames differently
   header <- c(" Period "," Adopters "," Cum Adopt. (%) ",
               " Hazard Rate "," Density ",
-              if (!skip.moran) c(" Moran's I (sd) ") else NULL
-              )
+              if (!skip.moran) c(" Moran's I (sd) ") else NULL)
 
   slen   <- nchar(header)
   hline  <- paste(sapply(sapply(slen, rep.int, x="-"), paste0, collapse=""),
@@ -247,44 +319,34 @@ summary.diffnet <- function(
   # Quick Formatting function
   qf <- function(x, digits=2) sprintf(paste0("%.",digits,"f"), x)
 
+  # Start printing result
   cat("Diffusion network summary statistics\n",
-      "Name     : ", meta$name, "\n",
-      "Behavior : ", meta$behavior, "\n",
-      rule,"\n",sep="")
-  cat(header,"\n")
-  cat(hline, "\n")
-  for (i in 1:nrow(out)) {
-    cat(sprintf(
-      paste0("%",slen,"s", collapse=" "),
-      qf(meta$pers[slices[i]],0), qf(out[i,1],0),
-      sprintf("%s (%s)",
-        qf(out$cum_adopt[i],0),
-        qf(out$cum_adopt_pcent[i])
-        ),
-      ifelse(i==1, "-",qf(out$hazard[i])), qf(out$density[i]),
-      if (!skip.moran) {
-        if (is.nan(out$moran_sd[i]))
-          " - "
-        else
-          sprintf("%s (%s) %-3s",
-                  qf(out$moran_obs[i]),
-                  qf(out$moran_sd[i]),
-                  ifelse(out$moran_pval[i] <= .01, "***",
-                         ifelse(out$moran_pval[i] <= .05, "**",
-                                ifelse(out$moran_pval[i] <= .10, "*", ""
-                              )))
-                )
-      } else ""
-    ), "\n")
+      "Name     : ", meta$name, "\n")
+
+  if (single) {
+    cat(" Behavior : ", meta$behavior[[1L]], "\n",
+        rule,"\n",sep="")
+    cat(header,"\n")
+    cat(hline, "\n")
+    summary_diffnet_out_display(out, slen, meta, slices, qf, skip.moran)
+    cat(rule, "\n",
+        paste("Left censoring  :", sprintf("%3.2f (%d)", lc/meta$n, lc)), "\n",
+        paste("Right centoring :", sprintf("%3.2f (%d)", rc/meta$n, rc)), "\n")
+  } else {
+    beh_names <- meta$behavior
+    for (q in 1:length(object$cumadopt)) {
+      cat("\n Behavior : ", beh_names[[q]], "\n",
+          rule,"\n",sep="")
+      cat(header,"\n")
+      cat(hline, "\n")
+      summary_diffnet_out_display(out_list[[q]], slen, meta, slices, qf, skip.moran)
+      cat(rule, "\n",
+          paste("Left censoring  :", sprintf("%3.2f (%d)", lc/meta$n, data_beh_list[[q]][1])), "\n",
+          paste("Right centoring :", sprintf("%3.2f (%d)", rc/meta$n, data_beh_list[[q]][2])), "\n")
+    }
   }
 
-
-  # print(out, digits=2)
-
   cat(
-    rule,
-    paste(" Left censoring  :", sprintf("%3.2f (%d)", lc/meta$n, lc)),
-    paste(" Right centoring :", sprintf("%3.2f (%d)", rc/meta$n, rc)),
     paste(" # of nodes      :", sprintf("%d",meta$n)),
     "\n Moran's I was computed on contemporaneous autocorrelation using 1/geodesic",
     " values. Significane levels  *** <= .01, ** <= .05, * <= .1.",
@@ -294,6 +356,33 @@ summary.diffnet <- function(
   invisible(out)
 }
 
+
+summary_diffnet_out_display <- function(out, slen, meta, slices, qf, skip.moran) {
+  for (i in 1:nrow(out)) {
+    cat(sprintf(
+      paste0("%",slen,"s", collapse=" "),
+      qf(meta$pers[slices[i]],0), qf(out[i,1],0),
+      sprintf("%s (%s)",
+              qf(out$cum_adopt[i],0),
+              qf(out$cum_adopt_pcent[i])
+      ),
+      ifelse(i==1, "-",qf(out$hazard[i])), qf(out$density[i]),
+
+      if (!skip.moran) {
+        if (is.nan(out$moran_sd[i])) {" - "}
+        else {sprintf("%s (%s) %-3s",
+                      qf(out$moran_obs[i]),
+                      qf(out$moran_sd[i]),
+                      ifelse(out$moran_pval[i] <= .01, "***",
+                             ifelse(out$moran_pval[i] <= .05, "**",
+                                    ifelse(out$moran_pval[i] <= .10, "*", ""
+                                    )))
+        )}
+      } else {""}
+    ), "\n")
+  }
+}
+
 #' Plot the diffusion process
 #'
 #' Creates a colored network plot showing the structure of the graph through time
@@ -656,7 +745,7 @@ plot_diffnet.default <- function(
 #' @return Invisible. A data frame with the calculated coordinates, including:
 #' `toa`, `threshold`, and `jit` (a jittered version of `toa`).
 #' @author George G. Vega Yon
-plot_threshold <- function(graph, expo, ...) UseMethod("plot_threshold")
+plot_threshold <- function(graph, expo,...) UseMethod("plot_threshold")
 
 #' @export
 #' @rdname plot_threshold
@@ -679,13 +768,21 @@ plot_threshold.diffnet <- function(graph, expo, ...) {
     args$toa <- graph$toa
   }
 
-  do.call(plot_threshold.default, c(list(graph = graph$graph, expo=expo), args))
+  if (length(args$vertex.label) == 0L)
+    args$vertex.label <- nodes(graph)
+
+  do.call(plot_threshold.default,
+          c(list(graph = graph$graph, expo = expo), args))
 }
 
 #' @export
 #' @rdname plot_threshold
 plot_threshold.array <- function(graph, expo, ...) {
-  plot_threshold.default(as_dgCMatrix(graph), expo = expo, ...)
+  plot_threshold.default(
+    as_dgCMatrix(graph),
+    expo = expo,
+    ...
+    )
 }
 
 #' @export
@@ -875,12 +972,15 @@ plot_threshold.default <- function(
 
   # Positioning labels can be harsh, so we try with this algorithm
   if (!length(vertex.label)) vertex.label <- 1:n
-  graphics::text(x=jit, y=y, labels = vertex.label,
-       pos = vertex.label.pos,
-       cex = vertex.label.cex,
-       col = vertex.label.color,
-       adj = vertex.label.adj
-       )
+  graphics::text(
+    x=jit,
+    y=y,
+    labels = vertex.label,
+    pos = vertex.label.pos,
+    cex = vertex.label.cex,
+    col = vertex.label.color,
+    adj = vertex.label.adj
+  )
 
   # par(oldpar)
 
