@@ -250,18 +250,21 @@ dgr.array <- function(graph, cmode, undirected, self, valued) {
 #' Calculates exposure to adoption over time via multiple different types of weight
 #' matrices.  The basic  model is exposure to adoption by immediate neighbors
 #' (outdegree) at the time period prior to ego’s adoption. This exposure can also be
-#' based on (1) incoming ties, (2) structural equivalence, (3) indirect ties, (4)
-#' attribute weighted (5) network-metric weighted (e.g., central nodes have more
-#' influence), and attribute-weighted (e.g., based on homophily or tie strength).
+#' based on (1) incoming ties, (2) structural equivalence, (3) indirect ties,
+#' (4) network-metric weighted (e.g., central nodes have more
+#' influence), and (5) attribute-weighted (e.g., based on homophily or tie strength).
 #'
 #' @templateVar valued TRUE
 #' @templateVar dynamic TRUE
 #' @templateVar self TRUE
 #' @template graph_template
-#' @param cumadopt \eqn{n\times T}{n * T} matrix. Cumulative adoption matrix obtained from
-#' \code{\link{toa_mat}}
+#' @param cumadopt \eqn{n\times T}{n * T} matrix for single diffusion.
+#' \eqn{n\times T \times Q}{n * T * Q} array for \eqn{Q}{Q} diffusion processes.
+#' Cumulative adoption matrix obtained from \code{\link{toa_mat}}
 #' @param attrs Either a character scalar (if \code{graph} is diffnet),
-#' or a numeric matrix of size \eqn{n\times T}{n * T}. Weighting for each time, period (see details).
+#' a numeric matrix of size \eqn{n\times T}{n * T}, or an array of size
+#' \eqn{n\times T \times Q}{n * T * Q} (only for multi diffusion).
+#'  Weighting for each time period (see details).
 #' @param alt.graph Either a graph that should be used instead of \code{graph},
 #' or \code{"se"} (see details).
 #' @param outgoing Logical scalar. When \code{TRUE}, computed using outgoing ties.
@@ -448,12 +451,46 @@ dgr.array <- function(graph, cmode, undirected, self, valued) {
 #' stopifnot(all(test[!is.na(test)]))
 #'
 #'
+#' # Examples for multi-diffusion ---------------------------
+#'
+#' # Running a multi-diffusion simulation, with q=2 behaviors
+#' set.seed(999)
+#' n <- 40; t <- 5; q <- 2;
+#' graph <- rgraph_ws(n, t, p=.3)
+#' seed_prop_adopt <- rep(list(0.1), q)
+#'
+#' diffnet <- rdiffnet(seed.graph = graph, t = t, seed.p.adopt = seed_prop_adopt)
+#'
+#' # Getting the cumulative adoption array of dims n x T x q
+#' cumadopt_2 <- diffnet$cumadopt  # list of matrices
+#' cumadopt_2 <- array(unlist(cumadopt_2), dim = c(n, t, q))
+#'
+#' expo2 <- exposure(diffnet$graph, cumadopt = cumadopt_2)
+#'
+#' # With an attribute --
+#'
+#' X <- matrix(runif(n * t), nrow = n, ncol = t) # matrix n x T
+#' ans3 <- exposure(diffnet$graph, cumadopt = cumadopt_2, attrs=X)
+#'
+#' X <- array(runif(n * t * q), dim = c(n, t, q)) # array n x T x q
+#' ans4 <- exposure(diffnet$graph, cumadopt = cumadopt_2, attrs=X)
+#'
+#' # Exposure based on Structural Equivalence --
+#'
+#' diffnet_1 <- split_behaviors(diffnet)[[1]]
+#' se <- struct_equiv(diffnet)
+#' se <- lapply(se, function(x) {
+#'   ans <- methods::as(x$SE, "dgCMatrix")
+#'     ans@x <- 1/(ans@x + 1e-20)
+#'       ans
+#'       })
+#' ans6 <- exposure(diffnet, cumadopt = cumadopt_2, alt.graph = se, valued=TRUE)
 #'
 #' @family statistics
 #' @keywords univar
 #' @return A matrix of size \eqn{n\times T}{n * T} with exposure for each node.
 #' @export
-#' @author George G. Vega Yon & Thomas W. Valente
+#' @author George G. Vega Yon, Thomas W. Valente, and Aníbal Olivera M.
 #' @name exposure
 NULL
 
@@ -476,15 +513,29 @@ NULL
   # Checking self
   if (!self) graph <- sp_diag(graph, rep(0, nnodes(graph)))
 
-  ans <- ( graph %*% (attrs * cumadopt) )
+  norm <- graph %*% attrs + 1e-20
 
-  if (normalized) as.vector(ans/( graph %*% attrs + 1e-20 ))
-  else as.vector(ans)
+  if (!is.na(dim(cumadopt)[3])) {
+    ans <- array(0, dim = c(dim(cumadopt)[1],dim(cumadopt)[3]))
+
+    for (q in 1:dim(cumadopt)[3]) {
+      if (normalized) {
+        ans[,q] <- as.vector(graph %*% (attrs * cumadopt[,,q]) / norm)
+      } else {
+        ans[,q] <- as.vector(graph %*% (attrs * cumadopt[,,q]))
+      }
+    }
+  } else {
+    ans <- graph %*% (attrs * cumadopt)
+
+    if (normalized) {
+      ans <- ans/ norm
+    }
+  }
+
+  return(as.vector(ans))
 }
 
-# library(microbenchmark)
-# microbenchmark(.exposure, netdiffuseR:::exposure_cpp)
-
 check_lags <- function(npers, lags) {
 
   # Checking length
@@ -552,7 +603,11 @@ exposure <- function(
 
   # Checking attrs
   if (!length(attrs)) {
-    attrs <- matrix(1, ncol=ncol(cumadopt), nrow=nrow(cumadopt))
+    if (!is.na(dim(cumadopt)[3])) {
+    attrs <- array(1, dim = c(nrow(cumadopt), ncol(cumadopt), 1))}
+    else {attrs <- matrix(1, ncol=ncol(cumadopt), nrow=nrow(cumadopt))}
+  } else if (!is.na(dim(cumadopt)[3])) {
+    attrs <- array(attrs, dim = c(nrow(attrs), ncol(attrs), 1))
   }
 
   # Checking alt graph
@@ -611,12 +666,19 @@ exposure.list <- function(
   # attrs can be either
   #  degree, indegree, outdegree, or a user defined vector.
   #  by default is user equal to 1
-  da <- dim(attrs)
-  if (!length(da)) stop("-attrs- must be a matrix of size n by T.")
-  if (any(da != dim(cumadopt))) stop("Incorrect size for -attrs-. ",
-                                     "It must be of size that -cumadopt-.")
 
+  # dim(attrs) default n x T matrix of 1's
+  if (!length(dim(attrs))) stop("-attrs- must be a matrix of size n by T.")
+
+  if (!is.na(dim(cumadopt)[3])) {
+    if (dim(cumadopt)[3]>1 && any(dim(attrs)[-3] != dim(cumadopt)[-3])) stop("Incorrect size for -attrs-. ",
+                                              "Does not match n dim or t dim.")
+  } else {
+    if (any(dim(attrs) != dim(cumadopt))) stop("Incorrect size for -attrs-. ",
+                                              "It must be of size that -cumadopt-.")
+  }
   add_dimnames.mat(cumadopt)
+  add_dimnames.mat(attrs)
 
   output <- exposure_for(graph, cumadopt, attrs, outgoing, valued, normalized,
                          self, lags)
@@ -637,17 +699,56 @@ exposure_for <- function(
   lags
   ) {
 
-  out <- matrix(nrow = nrow(cumadopt), ncol = ncol(cumadopt))
+  if (!is.na(dim(cumadopt)[3])) {
+    out <- array(NA, dim = c(dim(cumadopt)[1], dim(cumadopt)[2], dim(cumadopt)[3]))
 
-  if (lags >= 0L) {
-    for (i in 1:(nslices(graph) - lags))
-      out[,i+lags]<- .exposure(graph[[i]], cumadopt[,i,drop=FALSE], attrs[,i,drop=FALSE],
-                               outgoing, valued, normalized, self)
+    if (lags >= 0L) {
+      for (i in 1:(nslices(graph) - lags)) {
+        out[, i + lags, ] <- .exposure(graph[[i]],
+                                       cumadopt[, i, , drop = FALSE],
+                                       attrs[, i, , drop = FALSE],
+                                       outgoing = outgoing,
+                                       valued = valued,
+                                       normalized = normalized,
+                                       self = self)
+      }
+    } else {
+      for (i in (1 - lags):nslices(graph)) {
+        out[, i + lags, ] <- .exposure(graph[[i]],
+                                       cumadopt[, i, , drop = FALSE],
+                                       attrs[, i, , drop = FALSE],
+                                       outgoing = outgoing,
+                                       valued = valued,
+                                       normalized = normalized,
+                                       self = self)
+      }
+    }
   } else {
-    for (i in (1-lags):nslices(graph))
-      out[,i+lags]<- .exposure(graph[[i]], cumadopt[,i,drop=FALSE], attrs[,i,drop=FALSE],
-                               outgoing, valued, normalized, self)
+    out <- array(NA, dim = c(dim(cumadopt)[1], dim(cumadopt)[2]))
+
+    if (lags >= 0L) {
+      for (i in 1:(nslices(graph) - lags)) {
+        out[, i + lags] <- .exposure(graph[[i]],
+                                     cumadopt[, i, drop = FALSE],
+                                     attrs[, i, drop = FALSE],
+                                     outgoing = outgoing,
+                                     valued = valued,
+                                     normalized = normalized,
+                                     self = self)
+      }
+    } else {
+      for (i in (1 - lags):nslices(graph)) {
+        out[, i + lags] <- .exposure(graph[[i]],
+                                     cumadopt[, i, drop = FALSE],
+                                     attrs[, i, drop = FALSE],
+                                     outgoing = outgoing,
+                                     valued = valued,
+                                     normalized = normalized,
+                                     self = self)
+      }
+    }
   }
+
   return(out)
 }
 
