@@ -81,15 +81,15 @@ double st_mean(double y0, double y1) {return (y0 + y1) /2.0;}
 // XPtr<funcPtr> st_getfun(std::string funname) {
 void st_getfun(std::string funname, funcPtr & fun) {
   if      (funname == "distance")                           fun = &st_dist;
-  else if ((funname == "quaddist") | (funname == "^2"))       fun = &st_quaddist;
-  else if ((funname == "greater") | (funname == ">"))       fun = &st_greater;
-  else if ((funname == "greaterequal") | (funname == ">=")) fun =  &st_greaterequal;
-  else if ((funname == "smaller") | (funname == "<"))       fun =  &st_smaller;
-  else if ((funname == "smallerequal") | (funname == "<=")) fun =  &st_smallerequal;
-  else if ((funname == "equal") | (funname == "=="))        fun =  &st_equal;
-  else if ((funname == "min") | (funname == "minimum"))        fun =  &st_min;
-  else if ((funname == "max") | (funname == "maximum"))        fun =  &st_max;
-  else if ((funname == "mean") | (funname == "avg"))        fun =  &st_mean;
+  else if ((funname == "quaddist") || (funname == "^2"))       fun = &st_quaddist;
+  else if ((funname == "greater") || (funname == ">"))       fun = &st_greater;
+  else if ((funname == "greaterequal") || (funname == ">=")) fun =  &st_greaterequal;
+  else if ((funname == "smaller") || (funname == "<"))       fun =  &st_smaller;
+  else if ((funname == "smallerequal") || (funname == "<=")) fun =  &st_smallerequal;
+  else if ((funname == "equal") || (funname == "=="))        fun =  &st_equal;
+  else if ((funname == "min") || (funname == "minimum"))        fun =  &st_min;
+  else if ((funname == "max") || (funname == "maximum"))        fun =  &st_max;
+  else if ((funname == "mean") || (funname == "avg"))        fun =  &st_mean;
   else Rcpp::stop("Unkown function.");
 
   return ;
