@@ -1,5 +1,5 @@
 /*******************************************************************************
-* egonets.h header function for egonets.cpp
+* netdiffuser_extra.h header function for netdiffuser_extra.cpp
 *******************************************************************************/
 // [[Rcpp::depends(RcppArmadillo)]]
 #include <RcppArmadillo.h>
@@ -9,10 +9,6 @@
 
 using namespace Rcpp;
 
-arma::umat sparse_indexes(const arma::sp_mat & mat);
-
-double angle(double x0, double y0, double x1, double y1);
-
 arma::sp_mat sp_trimatl(const arma::sp_mat & x);
 
 int unif_rand_w_exclusion(int n, int e);
